﻿Type=Class
Version=2.71
@EndOfDesignText@
'Class module
Sub Class_Globals
	Type TextAlignType (Right As String, Left As String, Center As String)
	Type ControlType (IsLabel As String, IsTextBox As String, IsSpinner As String, IsSave As String, IsDelete As String, IsClone As String)
	Type InputTypes(Decimal As String, None As String, Numbers As String, Phone As String, Text As String)
	Public TextAlign As TextAlignType
	Public EditType As ControlType
	Public InputType As InputTypes
	Public Columns As Map
	Dim dummyView As View ' bug fix
	Public FontSize As Int
	Public Rows As Map
	Private numCols As Int
	Public RowHeight As Int
	Public cv As CustomListView 
	Public pnl As Panel
	Public cList As Map
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize()
	cList.Initialize 
	Columns.Initialize 
	Rows.Initialize 
	numCols = 0
	TextAlign.Initialize 
	TextAlign.Center = "Center"
	TextAlign.Left = "Left"
	TextAlign.Right = "Right"
	EditType.Initialize
	EditType.IsLabel = "Label"
	EditType.IsTextBox = "EditText"
	EditType.IsSpinner = "Spinner"
	EditType.IsSave = "Save"
	EditType.IsDelete = "Delete"
	EditType.IsClone = "Clone"
	InputType.Initialize 
	InputType.Decimal = "Decimal"
	InputType.None = "None"
	InputType.Numbers = "Numbers"
	InputType.Phone = "Phone"
	InputType.Text = "Text"	
	FontSize = 12
End Sub

' add a column to the grid
Public Sub AddColumn(ColumnName As String, Caption As String, Width As String, Format As String, Alignment As String, cEditType As String, cInputType As String)
	Dim m As Map
	m.Initialize
	m.Put("ColumnName", ColumnName)
	m.Put("Caption", Caption)
	m.Put("Width", Width)
	m.Put("Format", Format)
	m.Put("Alignment", Alignment)
	m.Put("EditType", cEditType)
	m.Put("InputType", cInputType)
	Columns.Put(ColumnName, m)
	numCols = numCols + 1
	Log("Column " & numCols & " Added")
End Sub

Public Sub AddColumnList(ColumnName As String, ColumnList As List)
	cList.Put(ColumnName, ColumnList)
End Sub

' add a row to the grid
Public Sub AddRow(rowID As String, rowItems() As String)
	Dim m As Map
	m.Initialize
	For i = 0 To Columns.Size - 1
		Dim sColumnName As String
		sColumnName = Columns.GetKeyAt(i)
		Select Case sColumnName
		Case "save", "delete", "clone"
		Case Else
			m.Put(sColumnName, rowItems(i))
		End Select
	Next
	Rows.Put(rowID, m)
	Log("Row " & Rows.size & " Added")
End Sub

' add a row to the grid
Public Sub AddRowMap(rowID As String, rowMap As Map)
	Rows.put(rowID, rowMap)
	Log("Row " & Rows.size & " Added")
End Sub

' remove a row using a rowid
Public Sub RemoveRow(rowID As String)
	If Rows.ContainsKey(rowID) Then Rows.Remove(rowID)  
End Sub

' add a save button
Public Sub AddSave()
	AddColumn("save", "", "48", "{0}", TextAlign.Center, EditType.IsSave, InputType.None)
End Sub

' add a delete button
Public Sub AddDelete()
	AddColumn("delete", "", "48", "{0}", TextAlign.Center, EditType.IsDelete, InputType.None)
End Sub

' add a clone button
Public Sub AddClone()
	AddColumn("clone", "", "48", "{0}", TextAlign.Center, EditType.IsClone, InputType.None)
End Sub

' draw the grid
Sub DrawGrid()
	Dim intL As Int: intL = 10
	Dim sColumName As String: sColumName = ""
	Dim sWidth As String: sWidth = ""
	Dim sCaption As String: sCaption = ""
	Dim sFormat As String: sFormat = ""
	Dim sAlignment As String: sAlignment = ""
	Dim cEditType As String: cEditType = ""
	Dim cInputType As String: cInputType = ""
	Dim strL As String = ""
	Dim intW As Int: intW = 0
	' add headers
	pnl.Color = Colors.black
	For i = 0 To Columns.Size - 1
		Dim cm As Map
		cm.Initialize 
		cm = Columns.GetValueAt(i) 
		sColumName = cm.Get("ColumnName")
		sCaption = cm.Get("Caption")
		sWidth = cm.Get("Width")
		sAlignment = cm.Get("Alignment")
		cEditType = cm.Get("EditType")
		intW = sWidth
		' build the headings
		Dim lbl As Label
		lbl.Initialize("h" & sColumName)
		Select Case sAlignment
		Case TextAlign.Center: lbl.Gravity = Bit.OR(Gravity.CENTER_VERTICAL, Gravity.CENTER)
		Case TextAlign.Left: lbl.Gravity = Bit.OR(Gravity.CENTER_VERTICAL, Gravity.LEFT)
		Case TextAlign.Right: lbl.Gravity = Bit.OR(Gravity.CENTER_VERTICAL, Gravity.RIGHT)
		End Select
		Select Case cEditType
		Case EditType.IsClone
		Case EditType.IsDelete
		Case EditType.IsSave
		Case Else
			lbl.Text = sCaption
		End Select
		lbl.TextSize = FontSize
		lbl.TextColor = Colors.White
		pnl.AddView(lbl, intL, 0dip, intW, 40)
		intL = intL + intW + 10
		Log(sColumName & "," & intW)
	Next
	' resize the controls, potrait mode cuts controls
	pnl.Width = intL
	cv.AsView.Width = intL
	Log("Headers Created")
	' addrows
	cv.clear
	For r = 0 To Rows.Size - 1
		' get row data
		Dim rowID As String
		Dim rd As Map
		rd.Initialize 
		rowID = Rows.GetKeyAt(r)
		rd = Rows.GetValueAt(r) 
		' the panel to hold everything
		Dim rp As Panel
		rp.Initialize("")
		rp.Color = Colors.Black
		' define each column
		intL = 10
		For i = 0 To Columns.Size - 1
			Dim cm As Map
			cm.Initialize 
			cm = Columns.GetValueAt(i) 
			sColumName = cm.Get("ColumnName")
			sWidth = cm.Get("Width")
			sFormat = cm.Get("Format")
			sAlignment = cm.Get("Alignment")
			cEditType = cm.Get("EditType")
			cInputType = cm.Get("InputType")
			intW = sWidth
			Log(sColumName & ", " & sWidth & ", " & sFormat & "," & sAlignment & "," & cEditType)
			Select Case cEditType
			Case EditType.IsClone
				Dim btn As Button
				btn.Initialize("clone")
				btn.SetBackgroundImage(LoadBitmap(File.DirAssets, "duplicate.png"))
				btn.Enabled = True
				btn.BringToFront 
				rp.AddView(btn, intL, 5, intW, 70)
			Case EditType.IsDelete
				Dim btn As Button
				btn.Initialize("delete")
				btn.SetBackgroundImage(LoadBitmap(File.DirAssets, "delete25.png"))
				btn.Enabled = True
				btn.BringToFront 
				rp.AddView(btn, intL, 5, intW, 70)
			Case EditType.IsSave
				Dim btn As Button
				btn.Initialize("save")
				btn.SetBackgroundImage(LoadBitmap(File.DirAssets, "save.png"))
				btn.Enabled = True
				btn.BringToFront 
				rp.AddView(btn, intL, 5, intW, 70)
			Case EditType.IsLabel
				Dim lbl As Label
				lbl.Initialize(sColumName)
				Select Case sAlignment
				Case TextAlign.Center: lbl.Gravity = Bit.OR(Gravity.CENTER_VERTICAL, Gravity.CENTER)
				Case TextAlign.Left: lbl.Gravity = Bit.OR(Gravity.CENTER_VERTICAL, Gravity.LEFT)
				Case TextAlign.Right: lbl.Gravity = Bit.OR(Gravity.CENTER_VERTICAL, Gravity.RIGHT)
				End Select
				lbl.Text = rd.Get(sColumName)
				lbl.TextSize = FontSize
				lbl.TextColor = Colors.white
				rp.AddView(lbl, intL, 5, intW, 70)
			Case EditType.IsTextBox
				Dim txt As EditText
				txt.Initialize(sColumName)
				Select Case sAlignment
				Case TextAlign.Center: txt.Gravity = Bit.OR(Gravity.CENTER_VERTICAL, Gravity.CENTER)
				Case TextAlign.Left: txt.Gravity = Bit.OR(Gravity.CENTER_VERTICAL, Gravity.LEFT)
				Case TextAlign.Right: txt.Gravity = Bit.OR(Gravity.CENTER_VERTICAL, Gravity.RIGHT)
				End Select
				txt.Text = rd.Get(sColumName)
				txt.TextSize = FontSize
				txt.TextColor = Colors.white
				txt.SingleLine = True
				Select Case cInputType
				Case InputType.Decimal: txt.InputType = txt.INPUT_TYPE_DECIMAL_NUMBERS 
				Case InputType.None: txt.InputType = txt.INPUT_TYPE_NONE 
				Case InputType.Numbers: txt.InputType = txt.INPUT_TYPE_NUMBERS 
				Case InputType.Phone: txt.InputType = txt.INPUT_TYPE_PHONE 
				Case InputType.Text: txt.InputType = txt.INPUT_TYPE_TEXT 
				End Select
				rp.AddView(txt, intL, 5, intW, 70)
			Case EditType.IsSpinner
				Dim cbo As Spinner
				cbo.Initialize(sColumName)
				' find out if we have items to select from for the combo
				'cbo.Add(rd.Get(sColumName)) 
				Dim ml As List
				ml.Initialize
				ml = cList.Get(sColumName) 
				If ml = Null Then
				Else
					For xi = 0 To ml.Size - 1
						cbo.Add(ml.Get(xi))
					Next
				End If
				cbo.SelectedIndex = cbo.IndexOf(rd.Get(sColumName))
				cbo.TextSize = FontSize
				cbo.TextColor = Colors.white
				rp.AddView(cbo, intL, 5, intW, 70)
			End Select
			intL = intL + intW + 10
		Next
		cv.Add(rp, RowHeight, rowID)
	Next
End Sub

' get the row as a map of the selected position
Sub GetRow(Index As Int) As Map
	Dim mOut As Map
	mOut.Initialize 
	Dim cEditType As String
	Dim cColumnName As String
	Dim pnl As Panel
	pnl = cv.GetPanel(Index)
	For i = 0 To Columns.Size - 1
		Dim cm As Map
		cm.Initialize 
		cm = Columns.GetValueAt(i) 
		cColumnName = cm.Get("ColumnName")
		cEditType = cm.Get("EditType")
		Select Case cEditType
		Case EditType.IsLabel
			Dim lbl As Label
			lbl = pnl.GetView(i)
			mOut.Put(cColumnName, lbl.Text)
		Case EditType.IsTextBox
			Dim txt As EditText
			txt = pnl.GetView(i)
			mOut.Put(cColumnName, txt.Text)
		Case EditType.IsSpinner
			Dim cbo As Spinner
			cbo = pnl.GetView(i)
			mOut.Put(cColumnName, cbo.SelectedItem)
		End Select
	Next
	Return mOut
End Sub
